<?php

    /*

        Plugin Name: 
        Description: Integrate your campaigns directly onto your WordPress-website!
        Version: 2.11
        
    */
    
    if (!defined('ABSPATH')) exit;

    $cc_config = [
        'PLUGIN_NAME' => '',
        'PLUGIN_VERSION' => '2.11',
        'API_ENDPOINT' => 'https:///api',
        'DATABASE_TABLE' => $wpdb->base_prefix . 'campaigns',
        'OPTION_NAMES' => [
            'TOKEN' => 'campaign_plugin_token'
        ],
        'VIEWS_DIR' => __DIR__ . '/views/',
        'ACTIONS_DIR' => __DIR__ . '/actions/',
        'CONTROLLERS_DIR' => __DIR__ . '/controllers/',
        'SITE' => get_current_blog_id(),
        'MULTISITE' => is_multisite()
    ];

    include __DIR__ . '/api.php';

    add_action('admin_menu', 'campaigns_admin_pages');
    add_action('admin_enqueue_scripts', 'campaigns_admin_sources');
    add_action('parse_request', 'campaigns_endpoint', 0);

    register_activation_hook(__FILE__, 'campaigns_enable');
    register_uninstall_hook(__FILE__, 'campaigns_uninstall');

    add_filter('site_transient_update_plugins', 'campaigns_update');
    add_filter('upgrader_source_selection', 'campaigns_change_source', 10, 4);
    add_action('upgrader_process_complete', 'campaigns_after_update', 10, 2);

    $cc_api = new CampaignsAPI([
        'config' => $cc_config
    ]);

    function campaigns_endpoint() {

        global $cc_api;
        global $cc_config;
        
        global $wp;
        global $wpdb;

        $slugs = explode(
            '/', 
            $wp->request
        );
        $slug = $slugs[0];

        $query = $wpdb->prepare('SELECT uuid FROM ' . $cc_config['DATABASE_TABLE'] . ' WHERE slug = "%s"', $slug);
        
        if ($cc_config['MULTISITE'] === true) {
            $query .= $wpdb->prepare(' AND site = %d', $cc_config['SITE']);
        }
        
        $uuid = $wpdb->get_var($query);

        if ($uuid !== null) {

            $project = $cc_api->single($uuid);

            if (($project !== null) && (!isset($project->error))) {
                
                $domains = $project->domains;

                $request = wp_remote_get(
                    'https://' . $domains[0]->domain . '/' . (isset($slugs[1]) ? $slugs[1] : ''), [
                        'httpversion' => '2.0',
                        'user-agent' => 'WordPressPlugin/' . $cc_config['PLUGIN_VERSION'] . ' (); ' . home_url(),
                        'timeout' => 10,
                        'sslverify' => false,
                        'headers' => [
                            'Plugin-Version' => $cc_config['PLUGIN_VERSION']
                        ]
                    ]
                );
                
                campaigns_response(
                    wp_remote_retrieve_body($request),
                    $slug,
                    $uuid
                );

            }
            else {

                echo 'Error: Campaign is no longer published or has been removed by author.';

            }

            exit;

        }

    }

    function campaigns_admin() {

        global $cc_api;
        global $cc_config;

        global $wp;
        global $wpdb;

        $actions = [
            'connect',
            'disconnect',
            'modify',
            'logout'
        ];

        $action = (isset($_GET['action'])  && in_array(sanitize_text_field($_GET['action']), $actions) ? $_GET['action'] : null);
        $method = $_SERVER['REQUEST_METHOD'];
        $token = get_option(
            $cc_config['OPTION_NAMES']['TOKEN']
        );

        if ($action !== null) {

            include $cc_config['ACTIONS_DIR'] . $action . '.php';

            echo '<script>window.location.replace("admin.php?page=campaigns");</script>';

        }

        if ($token !== false) {

            include $cc_config['CONTROLLERS_DIR'] . 'projects.php';

        }
        else {

            include $cc_config['CONTROLLERS_DIR'] . 'authenticate.php';

        }

    }

    function campaigns_admin_pages() {

        add_menu_page('', '', 'manage_options', 'campaigns', 'campaigns_admin', '', 4);
    
    }

    function campaigns_admin_sources() {

        wp_enqueue_style('custom_wp_admin_css', plugins_url('admin.css', __FILE__));

    }

    function campaigns_enable() {

        global $wpdb;

        $database = str_replace(
            [
                '[TABLE_NAME]', 
                '[TABLE_CHARSET]'
            ],
            [
                $wpdb->prefix . 'campaigns',
                $wpdb->get_charset_collate()
            ],
            file_get_contents(__DIR__ . '/database.sql')
        );
        
        $wpdb->query($database);

    }

    function campaigns_uninstall() {
        global $wpdb;
        global $cc_config;

        delete_option(
            $cc_config['OPTION_NAMES']['TOKEN']
        );

        $wpdb->query('DROP TABLE ' . $wpdb->prefix . 'campaigns');
    }
    
    function campaigns_response($body, $slug, $uuid) {
        
        global $cc_config;
        
        header('Content-Type: text/html; charset=utf-8');
        header('X-Plugin-Version: ' . $cc_config['PLUGIN_NAME'] . '/' . $cc_config['PLUGIN_VERSION']);
        header('X-Campaign-ID: ' . $uuid);

        preg_match_all('/<a.+(href ?= ?("|\')[^\2]*\2).*>.*<\/a>/U', $body, $matches);

        foreach ($matches[0] as $match) {
            
            if (strpos($match, 'data-custom="false"') !== false && strpos($match, 'target="_self"') !== false) {
                
                preg_match('/(?<=href=(\"|\'))[^\"\']+(?=(\"|\'))/', $match, $found);
                
                if (isset($found[0])) {
                    $href = $found[0];
                    $href = "/{$slug}{$href}";
                    $body = str_replace('href="' . $found[0] .'"', 'href="' . $href .'"', $body);
                }
                
            }
            
        }

        $body = str_replace('src="/bundle.client.js"', 'src="https://x.projects.webpages.one/bundle.client.js"', $body);

        echo $body;

        exit;
        
    }

    function campaigns_update($transient) {
        global $cc_config;

        if(empty($transient->checked)) {
            return $transient;
        }

        $transient_key = 'campaigns_upgrade_' . strtolower($cc_config['PLUGIN_NAME']);
        $request = $cc_config['MULTISITE'] === true ? get_site_transient($transient_key) : get_transient($transient_key);

        if(!$request) {
            $agency = json_decode(
                file_get_contents(__DIR__ . '/config.json')
            );

            $request = wp_remote_post(
                'https://wp.adpage.io/plugin/updater', [
                    'httpversion' => '2.0',
                    'sslverify' => true,
                    'body' => [
                        'agency' => $agency,
                        'version' => $cc_config['PLUGIN_VERSION'],
                        'server' => array_merge(
                            $_SERVER,
                            [
                                'PHP_VERSION' => phpversion()
                            ]
                        )
                    ]
                ]
            );

            if(!is_wp_error($request) && isset($request['response']['code']) && $request['response']['code'] == 200 && !empty($request['body'])) {
                if($cc_config['MULTISITE'] === true) {
                    set_site_transient($transient_key, $request, 43200); // 12 hours
                } else {
                    set_transient($transient_key, $request, 43200); // 12 hours
                }

            }
        }

        if($request && isset($request['body'])) {
            $response = json_decode($request['body']);

            if($response && $response->data->update_available === true && !empty($response->data->build->version) && !empty($response->data->build->path)) {
                $res = new stdClass();
                $res->slug = strtolower($cc_config['PLUGIN_NAME']);
                $res->plugin = plugin_basename(__FILE__);
                $res->new_version = $response->data->build->version;
                $res->package = $response->data->build->path;
                $transient->response[$res->plugin] = $res;
            }
        }

        return $transient;
    }

    function campaigns_change_source($source, $remote_source, $upgrader, $hook_extra) {

        if($hook_extra['plugin'] == plugin_basename(__FILE__)) {
            $new_source = ABSPATH . 'wp-content/upgrade/' . dirname(plugin_basename(__FILE__));
            rename($source, $new_source);

            $source = $new_source;
        }

        return $source;
    }

    function campaigns_after_update($upgrader, $options) {
        global $cc_config;

        if($options['action'] == ['update'] && $options['type'] === 'plugin') {
            $transient_key = 'campaigns_upgrade_' . strtolower($cc_config['PLUGIN_NAME']);

            if($cc_config['MULTISITE'] === true) {
                delete_site_transient($transient_key);
            } else {
                delete_transient($transient_key);
            }

            unlink(ABSPATH . 'wp-content/upgrade/' . dirname(plugin_basename(__FILE__)));
        }
    }